package application;

import javafx.event.EventHandler;
import javafx.scene.Cursor;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.shape.HLineTo;
import javafx.scene.shape.Line;
import javafx.scene.shape.LineTo;
import javafx.scene.shape.MoveTo;
import javafx.scene.shape.Path;
import javafx.scene.shape.Rectangle;
import javafx.scene.shape.VLineTo;

/**
 * @author David Brown
 * @version 2017-11-28
 *
 */
public class GraphicPane extends Pane {

    // private static final int BORDER_SIZE = 4;
    private static final int HANDLE_SIZE = 16;

    private Triangle model = null;

    Path triangle = new Path();
    MoveTo start = new MoveTo();
    HLineTo base = new HLineTo();
    VLineTo height = new VLineTo();
    LineTo hypotenuse = new LineTo();
    Line diagonal = new Line();
    Rectangle handle = new Rectangle();

    /**
     * @param model
     */
    public GraphicPane(final Triangle model) {
	this.model = model;
	this.layoutPane();
	this.registerListeners();
	this.getChildren().addAll(this.diagonal, this.triangle, this.handle);
    }

    /**
     *
     */
    private void layoutPane() {
	this.triangle.setStrokeWidth(5.0);
	this.triangle.setStroke(Color.BLUE);
	this.diagonal.setStrokeWidth(5.0);
	this.diagonal.setStroke(Color.RED);
	this.triangle.getElements().addAll(this.start, this.base, this.height,
		this.hypotenuse);
	this.handle.setWidth(HANDLE_SIZE);
	this.handle.setHeight(HANDLE_SIZE);
	this.handle.setFill(Color.BLACK);
    }

    /**
     *
     */
    private void registerListeners() {

	// Set the lower left corner starting point.
	this.start.setX(0);
	this.start.setY(0);
	// Define the base line.
	this.base.xProperty().bind(this.model.baseProperty()
		.multiply(this.widthProperty()).divide(Triangle.MAX_SIDE));
	// Define the height line.
	this.height.yProperty().bind(this.model.heightProperty()
		.multiply(this.heightProperty()).divide(Triangle.MAX_SIDE));
	// Define the hypotenuse line.
	this.hypotenuse.xProperty().bind(this.start.xProperty());
	this.hypotenuse.yProperty().bind(this.start.yProperty());
	// Define the rectangle location.
	this.handle.xProperty()
		.bind(this.base.xProperty().subtract(HANDLE_SIZE / 2));
	this.handle.yProperty()
		.bind(this.height.yProperty().subtract(HANDLE_SIZE / 2));

	// Define the mouse handling.
	handle.setOnMouseEntered(new EventHandler<MouseEvent>() {
	    @Override
	    public void handle(MouseEvent me) {
		GraphicPane.this.getScene().setCursor(Cursor.HAND);
	    }
	});

	handle.setOnMouseExited(new EventHandler<MouseEvent>() {
	    @Override
	    public void handle(MouseEvent me) {
		GraphicPane.this.getScene().setCursor(Cursor.DEFAULT);
	    }
	});

	// Define the diagonal.
	this.diagonal.startXProperty().set(0);
	this.diagonal.startYProperty().set(0);
	this.diagonal.endXProperty().bind(this.widthProperty());
	this.diagonal.endYProperty().bind(this.heightProperty());
    }

}
