package application;

import java.text.DecimalFormat;

import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.control.TextFormatter;
import javafx.scene.layout.VBox;
import javafx.util.StringConverter;
import javafx.util.converter.NumberStringConverter;

/**
 * @author David Brown
 * @version 2017-11-24
 */
public class NumericPane extends VBox {

    // ---------------------------------------------------------------
    private class IntegerFormatter extends StringConverter<Integer> {

	@Override
	public String toString(Integer object) {
	    String result = null;

	    if (object == null) {
		result = "0";
	    } else {
		result = object.toString();
	    }
	    return result;
	}

	@Override
	public Integer fromString(String string) {
	    return Integer.parseInt(string);
	}
    }

    // ---------------------------------------------------------------
    public NumericPane(Triangle model) {
	this.model = model;
	this.layoutPane();
	this.registerListeners();
    }

    private Triangle model = null;
    private TextField textFieldBase = new TextField();
    private TextField textFieldHeight = new TextField();
    private Label labelHypotenuse = new Label();
    /**
     * The format string for reading / displaying numeric input / output. Five
     * digits, one decimal.
     */
    private static final String formatString = "%5.1f";

    /**
     * Lays out the scene.
     */
    void layoutPane() {
	this.setAlignment(Pos.CENTER);
	this.setPadding(new Insets(25, 25, 25, 25));
	// Add elements.
	this.getChildren().add(new Label("Base:"));
	this.getChildren().add(textFieldBase);
	this.getChildren().add(new Label("Height:"));
	this.getChildren().add(textFieldHeight);
	this.getChildren().add(new Label("Hypotenuse:"));
	this.getChildren().add(labelHypotenuse);

	textFieldBase.setTextFormatter(
		new TextFormatter<Integer>(new IntegerFormatter()));
	textFieldHeight.setTextFormatter(
		new TextFormatter<Integer>(new IntegerFormatter()));
    }

    /**
     * Registers the listeners and handlers and bindings.
     */
    void registerListeners() {

	this.textFieldBase.textProperty().bindBidirectional(
		model.baseProperty(),
		new NumberStringConverter(new DecimalFormat()));

	this.textFieldHeight.textProperty().bindBidirectional(
		model.heightProperty(),
		new NumberStringConverter(new DecimalFormat()));

	// Bind the hypotenuse to the appropriate label.
	this.labelHypotenuse.textProperty()
		.bind(model.hypotenuseProperty().asString(formatString));
    }

}
