package application;

import javafx.beans.binding.DoubleBinding;
import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;

//---------------------------------------------------------------
/**
 * The right triangle model. Allows users to update the base and height
 * separately.
 *
 * @author David Brown from Byron Weber-Becker
 * @version 2017-11-23
 */
public class Triangle {

    /**
     * The maximum size of each of the two non-hypotenuse sides of the triangle.
     */
    public static final double MAX_SIDE = 100.0;

    public static void main(final String[] args) {
	final Triangle t = new Triangle();
	System.out.println("Base: " + t.baseProperty());
	System.out.println("Height: " + t.heightProperty());
	System.out.println("Hypotenuse: " + t.hypotenuseProperty());
	t.baseProperty().set(20);
	t.heightProperty().set(60);
	System.out.println("Base: " + t.baseProperty());
	System.out.println("Height: " + t.heightProperty());
	System.out.println("Hypotenuse: " + t.hypotenuseProperty());
    }

    /**
     * Sets the triangle base. Clamps the value between 0 and MAX_SIZE.
     */
    private final DoubleProperty base = new SimpleDoubleProperty(30.0) {

	@Override
	public void set(final double value) {
	    super.set(Math.min(Math.max(0, value), MAX_SIDE));
	}
    };

    /**
     * Sets the triangle height. Clamps the value between 0 and MAX_SIZE.
     */
    private final DoubleProperty height = new SimpleDoubleProperty(40.0) {

	@Override
	public void set(final double value) {
	    super.set(Math.min(Math.max(0, value), MAX_SIDE));
	}
    };

    /**
     * The hypotenuse based upon base and height.
     */
    private final DoubleBinding hypotenuse = new DoubleBinding() {
	// Call the super constructor to guarantee binding.
	{
	    super.bind(Triangle.this.base, Triangle.this.height);
	}

	@Override
	protected double computeValue() {
	    return Math.hypot(Triangle.this.base.get(),
		    Triangle.this.height.get());
	}
    };

    // ---------------------------------------------------------------
    /**
     * Returns the model's base property.
     *
     * @return base.
     */
    public DoubleProperty baseProperty() {
	return this.base;
    }

    /**
     * @return value of base.
     */
    public double getBase() {
	return this.base.get();
    }

    /**
     * @return value of height.
     */
    public double getHeight() {
	return this.height.get();
    }

    /**
     * @return value of hypotenuse.
     */
    public double getHypotenuse() {
	return this.hypotenuse.get();
    }

    // ---------------------------------------------------------------
    /**
     * Returns the model's height property.
     *
     * @return height.
     */
    public DoubleProperty heightProperty() {
	return this.height;
    }

    // ---------------------------------------------------------------
    /**
     * Returns the model's hypotenuse property.
     *
     * @return hypotenuse.
     */
    public DoubleBinding hypotenuseProperty() {
	return this.hypotenuse;
    }

    /**
     * @param base
     */
    public void setBase(final double base) {
	this.base.set(base);
    }

    /**
     * @param height
     */
    public void setHeight(final double height) {
	this.height.set(height);
    }

    // ---------------------------------------------------------------
}